@@ -69,8 +69,11 @@ map.poly <- function(database, regions = ".", exact = FALSE,
 ## But here you have a simple set of named polygons, not a map database.
 ## If it comes from anything but "world", we could break something else.
 ## We just leave it for now.
-      regexp <- paste("(^", regions, ")", sep = "", collapse = "|")
-        i <- grep(regexp, the.map$names, ignore.case = TRUE, perl=TRUE)
+        regexp <- paste("(^", regions, ")", sep = "", collapse = "|")
+## FIXME: find a better solution
+# perl regex crashes on >30000 characters, e.g. whole world database
+        i <- grep(regexp, the.map$names, ignore.case = TRUE,
+                  perl = (length(regions) < 1000) & (nchar(regexp) < 30000) )
       }
       if (length(i) == 0) stop("no recognized region names")
       nam <- the.map$names[i]
@@ -123,7 +126,8 @@ function(database = "world", regions = ".", exact = FALSE,
 	 col = 1, plot = TRUE, add = FALSE, namesonly = FALSE, 
          xlim = NULL, ylim = NULL, wrap = FALSE,
          resolution = if (plot) 1 else 0, type = "l", bg = par("bg"),
-         mar = c(4.1, 4.1, par("mar")[3], 0.1), myborder = 0.01, namefield="name", ...)
+         mar = c(4.1, 4.1, par("mar")[3], 0.1), myborder = 0.01, 
+         namefield="name", lforce = "n", ...)
 {
   # parameter checks
   if (resolution>0 && !plot) 
@@ -132,6 +136,9 @@ function(database = "world", regions = ".", exact = FALSE,
     stop("one of boundary and interior must be TRUE")
   doproj <- !missing(projection) || !missing(parameters) || !missing(
           orientation)
+  if (doproj && !requireNamespace("mapproj", quietly=TRUE)) {
+    stop("Please install the package 'mapproj' for projections.")
+  }
   coordtype <- maptype(database)
   if (coordtype == "unknown") 
      stop("missing database or unknown coordinate type")
@@ -143,9 +150,19 @@ function(database = "world", regions = ".", exact = FALSE,
   coord <- map.poly(database, regions, exact, xlim, ylim, 
                     boundary, interior, fill, as.polygon, namefield=namefield)
   if (is.na(coord$x[1])) stop("first coordinate is NA.  bad map data?")
+  if (length(wrap)>=2) {
+    antarctica <- if (length(wrap) == 2) -89.9 else wrap[3]
+    coord <- map.wrap.poly(coord, xlim=wrap[1:2], poly=fill, antarctica=antarctica)
+  }
+  # we can enforce xlim & ylim exactly
+  # this changes the output
+  if (lforce=="e") {
+    coord <- map.clip.poly(coord, xlim, ylim, poly=fill)
+  }
   if (plot) {
     .map.range(coord$range)
   }
+
   if (doproj) {
     nam <- coord$names
     coord <- mapproj::mapproject(coord, projection = projection,
@@ -153,15 +170,25 @@ function(database = "world", regions = ".", exact = FALSE,
     coord$projection = projection
     coord$parameters = parameters
     coord$orientation = orientation
+    coord$names <- nam
+    if (!is.null(xlim) && !is.null(ylim) && lforce %in% c("s","l")) {
+      prange <- mapproj::mapproject(x=rep(xlim,2), y=rep(ylim, each=2))
+      if (lforce=="s") {
+        xlim <- c(max(prange$x[c(1,3)]),min(prange$x[c(2,4)]))
+        ylim <- c(max(prange$y[c(1,2)]),min(prange$y[c(3,4)]))
+      } else {
+        xlim <- c(min(prange$x[c(1,3)]),max(prange$x[c(2,4)]))
+        ylim <- c(min(prange$y[c(1,2)]),max(prange$y[c(3,4)]))
+      }
+    }
     if (plot && coord$error)
       if (all(is.na(coord$x)))
         stop("projection failed for all data")
       else warning("projection failed for some data")
-    coord$names <- nam
   }
   # AD: we do wrapping first: slightly better than when run after the thinning
   #     also now the output data is also wrapped if plot=FALSE
-  if (wrap) coord <- map.wrap(coord)
+  if (length(wrap)==1 && wrap) coord <- map.wrap(coord)
   # do the plotting, if requested
   if (plot) {
     # for new plots, set up the coordinate system;
@@ -171,10 +198,10 @@ function(database = "world", regions = ".", exact = FALSE,
     if (!add) {
       opar = par(bg = bg)
       if (!par("new")) plot.new()
-      # xlim, ylim apply before projection
-      if (is.null(xlim) || doproj) xrange <- range(coord$x, na.rm = TRUE)
+      # xlim, ylim apply before projection unless we have pforce=TRUE
+      if (is.null(xlim) || (doproj && !(lforce %in% c("s","l")))) xrange <- range(coord$x, na.rm = TRUE)
       else xrange <- xlim
-      if (is.null(ylim) || doproj) yrange <- range(coord$y, na.rm = TRUE)
+      if (is.null(ylim) || (doproj && !(lforce %in% c("s","l")))) yrange <- range(coord$y, na.rm = TRUE)
       else yrange <- ylim
       if (coordtype != "spherical" || doproj) {
 	aspect <- c(1, 1) 
